# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributors:
# Dragos Chirila, Finsiel Romania
#
# This file contains code taken and adapted from the CookieCrumbler product 
# (http://zope.org/Members/hathawsh/CookieCrumbler), which is released under
# Zope Public Licence (ZPL) Version 2.0 by Zope Corporation.
# You can obtain a copy of ZPL at http://www.zope.com/Legal/zpl_2
#

""" CookieCrumbler """

# Python imports
from os import path
from urllib import quote, unquote
from base64 import encodestring

# Zope imports
from DateTime import DateTime
from ZPublisher import BeforeTraverse
from ZPublisher.HTTPRequest import HTTPRequest
from AccessControl import ClassSecurityInfo
import Globals

# Product imports

# Constants
ATTEMPT_DISABLED = -1
ATTEMPT_NONE = 0
ATTEMPT_LOGIN = 1
ATTEMPT_CONT = 2

class BWCookieCrumbler:
    """ """

    def __init__(self):
        """ """
        pass

    auth_cookie = '__bw'
    name_cookie = '__bw_name'
    pw_cookie = '__bw_password'
    persist_cookie = '__bw_persistent'
    auto_login_page = 'login_html'
    logout_page = 'logout_html'

    security = ClassSecurityInfo()

    security.declarePrivate('delRequestVar')
    def delRequestVar(self, req, name):
        try: del req.other[name]
        except: pass
        try: del req.form[name]
        except: pass
        try: del req.cookies[name]
        except: pass
        try: del req.environ[name]
        except: pass

    # Allow overridable cookie set/expiration methods.
    security.declarePrivate('getCookieMethod')
    def getCookieMethod( self, name='setAuthCookie', default=None ):
        return getattr( self.aq_inner.aq_parent, name, default )

    security.declarePrivate('setDefaultAuthCookie')
    def defaultSetAuthCookie( self, resp, cookie_name, cookie_value ):
        resp.setCookie( cookie_name, cookie_value, path='/')

    security.declarePrivate('defaultExpireAuthCookie')
    def defaultExpireAuthCookie( self, resp, cookie_name ):
        resp.expireCookie( cookie_name, path='/')

    security.declarePrivate('modifyRequest')
    def modifyRequest(self, req, resp):
        # Returns flags indicating what the user is trying to do.
        if req.__class__ is not HTTPRequest:
            return ATTEMPT_DISABLED

        if not req[ 'REQUEST_METHOD' ] in ( 'GET', 'PUT', 'POST' ):
            return ATTEMPT_DISABLED

        if not req._auth:
            if (req.has_key(self.pw_cookie) and
                req.has_key(self.name_cookie)):
                # Attempt to log in and set cookies.
                name = req[self.name_cookie]
                pw = req[self.pw_cookie]
                ac = encodestring('%s:%s' % (name, pw))
                req._auth = 'basic %s' % ac
                resp._auth = 1
                if req.get(self.persist_cookie, 0):
                    # Persist the user name (but not the pw or session)
                    expires = (DateTime() + 365).toZone('GMT').rfc822()
                    resp.setCookie(self.name_cookie, name, path='/',
                                   expires=expires)
                else:
                    # Expire the user name
                    resp.expireCookie(self.name_cookie, path='/')
                method = self.getCookieMethod( 'setAuthCookie'
                                             , self.defaultSetAuthCookie )
                method( resp, self.auth_cookie, quote( ac ) )
                self.delRequestVar(req, self.name_cookie)
                self.delRequestVar(req, self.pw_cookie)
                return ATTEMPT_LOGIN
            elif req.has_key(self.auth_cookie):
                # Copy __ac to the auth header.
                ac = unquote(req[self.auth_cookie])
                req._auth = 'basic %s' % ac
                resp._auth = 1
                self.delRequestVar(req, self.auth_cookie)
                return ATTEMPT_CONT
        return ATTEMPT_NONE

    def __call__(self, container, req):
        '''The __before_publishing_traverse__ hook.'''
        resp = self.REQUEST['RESPONSE']
        attempt = self.modifyRequest(req, resp)
        if attempt == ATTEMPT_DISABLED:
            return
        if self.auto_login_page:
            if not req.get('disable_cookie_login__', 0):
                if (attempt == ATTEMPT_LOGIN or
                    not getattr(resp, '_auth', 0)):
                    page = getattr(container, self.auto_login_page, None)
                    if page is not None:
                        # Provide a login page.
                        req._hold(ResponseCleanup(resp))
                        resp.unauthorized = self.unauthorized
                        resp._unauthorized = self._unauthorized
        if attempt:
            phys_path = self.getPhysicalPath()
            if self.logout_page:
                # Cookies are in use.
                page = getattr(container, self.logout_page, None)
                if page is not None:
                    # Provide a logout page.
                    req._logout_path = phys_path + ('logout',)
            req._credentials_changed_path = (
                phys_path + ('credentialsChanged',))

    security.declarePublic('credentialsChanged')
    def credentialsChanged(self, user, name, pw):
        ac = encodestring('%s:%s' % (name, pw))
        method = self.getCookieMethod( 'setAuthCookie'
                                       , self.defaultSetAuthCookie )
        resp = self.REQUEST['RESPONSE']
        method( resp, self.auth_cookie, quote( ac ) )

    def _cleanupResponse(self):
        resp = self.REQUEST['RESPONSE']
        try: del resp.unauthorized
        except: pass
        try: del resp._unauthorized
        except: pass
        return resp

    security.declarePrivate('unauthorized')
    def unauthorized(self):
        url = self.getLoginURL()
        if url:
            raise 'Redirect', url
        # Use the standard unauthorized() call.
        resp = self._cleanupResponse()
        resp.unauthorized()

    def _unauthorized(self):
        url = self.getLoginURL()
        if url:
            resp = self.REQUEST['RESPONSE']
            resp.redirect(url, lock=1)
            # We don't need to raise an exception.
            return
        # Use the standard _unauthorized() call.
        resp = self._cleanupResponse()
        resp._unauthorized()

    security.declarePublic('getLoginURL')
    def getLoginURL(self):
        ''' Redirects to the login page. '''
        if self.auto_login_page:
            req = self.REQUEST
            resp = req['RESPONSE']
            iself = getattr(self, 'aq_inner', self)
            parent = getattr(iself, 'aq_parent', None)
            page = getattr(parent, self.auto_login_page, None)
            if page:
                retry = getattr(resp, '_auth', 0) and '1' or ''
                url = '%s?came_from=%s&retry=%s' % (page.absolute_url(), quote(req['URL']), retry)
                return url
        return None

    security.declarePublic('logged_in')
    def logged_in(self, REQUEST):
        """ """
        came_from = REQUEST.get('came_from', '')
        if REQUEST.AUTHENTICATED_USER.getUserName() == 'Anonymous User':
            if came_from:
                return REQUEST.RESPONSE.redirect(self.auto_login_page + '?came_from=' + quote(came_from) + '&failed=1')
            else:
                return REQUEST.RESPONSE.redirect(self.auto_login_page + '?failed=1')
        else:
            if came_from:
                return REQUEST.REPONSE.redirect(came_from)
            else:
                #redirect in expert area
                return REQUEST.RESPONSE.redirect(self.absolute_url(0) + '/Experts/' + REQUEST.AUTHENTICATED_USER.getUserName() + '/answerquestions_html')

    security.declarePublic('logout')
    def logout(self):
        ''' Logs out the user and redirects to the logout page. '''
        req = self.REQUEST
        resp = req['RESPONSE']
        method = self.getCookieMethod('expireAuthCookie', self.defaultExpireAuthCookie )
        method( resp, cookie_name=self.auth_cookie )
        redir = 0
        if self.logout_page:
            iself = getattr(self, 'aq_inner', self)
            parent = getattr(iself, 'aq_parent', None)
            page = getattr(parent, self.logout_page, None)
            if page is not None:
                redir = 1
                resp.redirect(page.absolute_url())
        if not redir:
            # Should not normally happen.
            return 'Logged out.'

    # Installation and removal of traversal hooks.
    def manage_beforeDelete(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            BeforeTraverse.unregisterBeforeTraverse(container, handle)

    def manage_afterAdd(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            container = container.this()
            nc = BeforeTraverse.NameCaller(self.getId())
            BeforeTraverse.registerBeforeTraverse(container, nc, handle)

Globals.InitializeClass(BWCookieCrumbler)

class ResponseCleanup:
    def __init__(self, resp):
        self.resp = resp

    def __del__(self):
        # Free the references.
        try: del self.resp.unauthorized
        except: pass
        try: del self.resp._unauthorized
        except: pass
        try: del self.resp
        except: pass