# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania


"""Categories are basic container objects that provide a standard
    interface for object management. Category objects also implement
    a management interface and can have arbitrary properties.
"""

#$Id: Category.py,v 1.25 2004/01/08 15:55:14 finrocvs Exp $

# Python imports
import time

# Zope imports
import DateTime
from Products.ZCatalog.CatalogAwareness import CatalogAware
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens, view
from OFS.Folder import Folder
from Products import meta_types

# Product imports
from CatalogManager import CatalogManager
import Question
from toutf8 import toUTF8
import Resources

manage_addCategoryForm=DTMLFile('dtml/categoryAdd', globals())
def manage_addCategory(self, id, title, description, enroledExperts=[], keywords='', CId='', creationDate='', REQUEST=None):
    """Add a new Category object """
    enroledExperts = self.utConvertToList(enroledExperts)
    keywords = self.utConvertLinesToList(keywords)
    if CId == '':
        CId = str(time.time())
    try: creationDate = DateTime.DateTime(creationDate)
    except: creationDate = DateTime.DateTime()
    ob = Category(id, title, description, keywords, enroledExperts, CId, creationDate)
    self._setObject(id, ob)
    ob = self._getOb(id)
    ob.updateCategoryEnroledExperts([], enroledExperts)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class Category(CatalogAware, Folder, toUTF8):
    """ Categories are the basic container objects and are analogous to directories. """

    meta_type = Resources.METATYPE_EXPERT_CATEGORY
    icon = 'misc_/BesserWisser/category.gif'

    security = ClassSecurityInfo()

    manage_options = (
        Folder.manage_options[:2]
        +
        (
            {'label' : 'Manage Properties', 'action' : 'manage_properties'},
        )
        +
        Folder.manage_options[3:]
    )

    option_tabs = (
        {'label':'Summary', 'action': 'index_html'},
        {'label':'Ask a question', 'action':'listexperts_html'},
        {'label':'View answers', 'action':'listanswers_html'},
        {'label':'Search', 'action':'search_html'}
    )

    # Define constructors inside the class so categories can be added hierarchically
    manage_addCategoryForm = manage_addCategoryForm
    manage_addCategory = manage_addCategory

    def __init__(self, id, title, description, keywords, enroledExperts, CId, creationDate):
        """ Category constructor"""
        self.id = id
        self.title = title
        self.description = description
        self.keywords = keywords
        self.enroledExperts = enroledExperts
        self.CId = CId
        self.creationDate = creationDate
        toUTF8.__dict__['__init__'](self)

    def all_meta_types( self, interfaces=None ):
        """Checks if the legal products are actually installed in Zope """
        y = [
            {'name': Resources.METATYPE_EXPERT_CATEGORY, 'action': 'manage_addCategoryForm'},
            {'name': Resources.METATYPE_EXPERT_QUESTION, 'action': 'manage_addQuestionForm'}
        ]
        additional_meta_types = ['Image', 'LDAPUserFolder', 'User Folder']
        for x in meta_types:
            if x['name'] in additional_meta_types:
                y.append(x)
        return y

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """ This method is called, whenever _setObject in ObjectManager gets
        called. This is the case after a normal add and if the object is a
        result of cut-paste- or rename-operation"""
        Folder.inheritedAttribute('manage_afterAdd')(self, item, container)
        self.catalogBesserWisserObject(self)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        """ This method is called, when the object is deleted."""
        Folder.inheritedAttribute('manage_beforeDelete')(self, item, container)
        self.updateCategoryEnroledExperts(self.enroledExperts, [])
        self.uncatalogBesserWisserObject(self)

    security.declareProtected('Change Categories', 'manage_editCategory')
    def manage_editCategory(self, CId='', title='', description='', enroledExperts=[], keywords='', REQUEST=None):
        """ Manage the edited values """
        self.title = title
        try: self.CId = CId
        except: self.CId = str(time.time())
        old_enroledExperts = self.enroledExperts
        new_enroledExperts = self.utConvertToList(enroledExperts)
        self.enroledExperts = new_enroledExperts
        self.updateCategoryEnroledExperts(old_enroledExperts, new_enroledExperts)
        self.keywords = self.utConvertLinesToList(keywords)
        self.description = description
        # update BesserWisser catalog
        self.recatalogBesserWisserObject(self)
        if REQUEST is not None:
            return MessageDialog(title = 'Edited', message = "The properties of %s have been changed!" % self.id, action = './manage_main',)

    security.declarePrivate('updateCategoryEnroledExperts')
    def updateCategoryEnroledExperts(self, old_enroledExperts, new_enroledExperts):
        """ Add this expert to each category in expertise list """
        add_enroledExperts = self.utListDifference(new_enroledExperts, old_enroledExperts)
        del_enroledExperts = self.utListDifference(old_enroledExperts, new_enroledExperts)
        for item in add_enroledExperts:
            try:
                expert = self.getEnroledExpert(item)
                expert.Expertise.append(self.absolute_url(1))
                expert._p_changed = 1
            except:
                pass
        for item in del_enroledExperts:
            try:
                expert = self.getEnroledExpert(item)
                expert.Expertise.remove(self.absolute_url(1))
                expert._p_changed = 1
            except:
                pass

    security.declarePublic('getCategory')
    def getCategory(self):
        """ Returns this """
        return self

    security.declarePublic('getCategoryQuestions')
    def getCategoryQuestions(self):
        """ Returns the list of questions in this category """
        return self.objectValues(Resources.METATYPE_EXPERT_QUESTION)

    security.declarePrivate('_recurse_experts')
    def _recurse_experts(self, elist):
        """ Add experts to list if they are not there already,
            and recurse into subcategories """
        l_experts_ids = self.getExpertsFolder().objectIds(Resources.METATYPE_EXPERT)
        for expert in self.enroledExperts:
            if expert and expert in l_experts_ids and expert not in elist:
                elist.append(expert)
        for object in self.objectValues(Resources.METATYPE_EXPERT_CATEGORY):
            object._recurse_experts(elist)

    security.declarePublic('experts_in_hierarchy')
    def experts_in_hierarchy(self):
        """ Return a list of expert ids in the entire hierarchy of categories """
        elist = []
        self._recurse_experts(elist)
        return elist

    security.declarePrivate('getParentCategories')
    def getParentCategories(self):
        """ Return a list with all parent categories """
        l_path = self.absolute_url(1)
        l_parts = l_path.split('/')
        l_trail = []
        l_part_path = ''
        for l_part in l_parts:
            l_part_path = l_part_path + '/' + l_part
            l_part_obj = self.unrestrictedTraverse(l_part_path)
            if l_part_obj.meta_type == Resources.METATYPE_EXPERT_CATEGORY:
                l_trail.append(l_part_obj)
        return l_trail

    security.declarePublic('getEncodingFromLanguage')
    def getEncodingFromLanguage(self, language):
        """ returns the encoding for a language """
        return self.getLanguageEncoding(language)

    security.declarePublic('searchQuestions')
    def searchQuestions(self, p_keyword):
        """ Search Catalog defined indexes 
            Returns the Expert Questions that correspond to the query string """
        if p_keyword != '':
            return self.searchQuestionsInCategory(p_keyword, self.absolute_url(1))
        else:
            return []

    security.declarePublic('searchQuestionsOfExpert')
    def searchQuestionsOfExpert(self, ExpertId):
        """ Search Catalog for questions asked to a certain expert
            Returns the Expert Questions that correspond to the query string """
        return self.getCatalogSearchItems({'ExpertId':ExpertId})

    # Subobjects
    security.declareProtected(view, 'manage_addQuestionForm')
    manage_addQuestionForm = Question.manage_addQuestionForm

    security.declareProtected(view, 'manage_addQuestion')
    manage_addQuestion = Question.manage_addQuestion

    security.declareProtected('Change Categories', 'manage_properties')
    manage_properties = DTMLFile('dtml/Category_properties',globals())

    security.declareProtected(view, 'index_html')
    index_html = DTMLFile('dtml/Category_view',globals())

    security.declareProtected(view, 'listexperts_html')
    listexperts_html = DTMLFile('dtml/Category_listExperts',globals())

    security.declareProtected(view, 'listanswers_html')
    listanswers_html = DTMLFile('dtml/Category_listQA',globals())

    security.declareProtected(view, 'search_html')
    search_html = DTMLFile('dtml/Category_searchQA',globals())

    security.declareProtected(view, 'askquestion_html')
    askquestion_html = DTMLFile('dtml/Category_askQuestion',globals())

    security.declareProtected(view, 'thank_you')
    thank_you = DTMLFile('dtml/Category_thankyou',globals())

InitializeClass(Category)
