# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania


"""Expert object
    Experts are users kept in a Experts Folder. Inquireres can only ask a question to an Expert.
"""

# Python imports

# Zope imports
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens, view
from OFS.SimpleItem import SimpleItem
import Globals
from DateTime import DateTime
from OFS.content_types import guess_content_type

# Product imports
from ExpertData import ExpertData
from Utils import Utils
from BWCookieCrumbler import BWCookieCrumbler
import Resources

_EXPERT_ROLE = "Expert"

manage_addExpertForm = Globals.DTMLFile('dtml/Expert_Step1_add', globals())
def manage_validateValuesStep1(self, REQUEST=None):
    """ Validate values from the forms and return to the form if there are errors """
    l_errors = []
    #load ExpertFormData object
    l_expertformdata = self.getSessionExpertFormData(self.getEmptyExpertFormData())
    #load form data
    l_Firstname = REQUEST.get('Firstname', '')
    l_Lastname = REQUEST.get('Lastname', '')
    l_Email =REQUEST.get('Email', '')
    l_ProfileNativeIsIn =REQUEST.get('ProfileNativeIsIn', '')
    l_LanguagesUnderstood = self.utConvertToList(REQUEST.get('LanguagesUnderstood', []))
    l_Nationality = REQUEST.get('Nationality', '')
    l_name = REQUEST.get('expert_name', '')
    l_password = REQUEST.get('expert_password', '')
    l_confirmpassword = REQUEST.get('confirmpassword', '')
    #validate form data
    if not l_Firstname:
        l_errors.append(Resources.ERR_MSG_FIRSTNAME_REQUIRED)
    if not l_Lastname:
        l_errors.append(Resources.ERR_MSG_LASTNAME_REQUIRED)
    if not l_Email:
        l_errors.append(Resources.ERR_MSG_EMAIL_REQUIRED)
    if self.existsExpertEmail(l_Email):
        l_errors.append(Resources.ERR_MSG_EMAIL_EXISTS)
    if not l_name:
        l_errors.append(Resources.ERR_MSG_USERID_REQUIRED)
    if l_name and not self.utLookupForUser(l_name):
        l_errors.append(Resources.ERR_MSG_USERID_EXISTS)
    if not l_password:
        l_errors.append(Resources.ERR_MSG_PASSWORD_REQUIRED)
    else:
        if l_password != l_confirmpassword:
            l_errors.append(Resources.ERR_MSG_PASSWORDS_MUST_MATCH)
    #load form data in ExpertFormData
    l_expertformdata.update(l_Firstname, l_Lastname, l_Email, l_ProfileNativeIsIn, l_LanguagesUnderstood, l_Nationality, l_name, l_password, [], '', '', '', 0, 0, None)
    #check for errors
    if len(l_errors)>0:
        l_errors.append(Resources.ERR_MSG_RETYPE_PASSWORDS)
        if REQUEST:
            self.setSessionExpertFormData(l_expertformdata)
            self.setSessionErrors(l_errors)
            return REQUEST.RESPONSE.redirect('manage_addExpertForm')
        else:
            return None
    else:
        self.setSessionExpertFormData(l_expertformdata)
        self.delSessionErrors()
        if REQUEST:
            return REQUEST.RESPONSE.redirect('manage_addExpertStep2')

manage_addExpertStep2 = Globals.DTMLFile('dtml/Expert_Step2_add', globals())
def manage_addExpert(self, name='', password='', Firstname='', Lastname='', Nationality='', Email='',
        ProfileEnglish='', ProfileNative='', Employer='', Expertise=[], LanguagesUnderstood=[],
        QuestionsPerDay=5, ProfileNativeIsIn='', isOrganisation=0, Picture='',
        onVacation=0, ExpectedReturnDate=None,
        REQUEST=None):
    """ Adds an Expert object 
        If this is called from a form, take the parameters from the session,
        otherwise take the arguments
    """
    #load ExpertFormData object
    l_expertformdata = self.getSessionExpertFormData(None)
    if l_expertformdata is not None:
        #load data from ExpertFormData
        Firstname = l_expertformdata.Firstname
        Lastname = l_expertformdata.Lastname
        Email = l_expertformdata.Email
        ProfileNativeIsIn = l_expertformdata.ProfileNativeIsIn
        LanguagesUnderstood = l_expertformdata.LanguagesUnderstood
        Nationality = l_expertformdata.Nationality
        name = l_expertformdata.name
        password = l_expertformdata.password
    Expertise = self.utConvertToList(Expertise)
    #create expert
    ob = Expert(name, password, Firstname, Lastname, Email, ProfileNativeIsIn,
        LanguagesUnderstood, Nationality, ProfileEnglish, ProfileNative, Employer,
        Expertise, QuestionsPerDay, isOrganisation, Picture,
        onVacation, ExpectedReturnDate)
    self._setObject(name, ob)
    ob = self._getOb(name)
    ob.updateExpertExpertise([], Expertise)

    self.acl_users._addRole(_EXPERT_ROLE)
    self.manage_role(_EXPERT_ROLE, permissions=['Access contents information', view])
    self.acl_users._addUser(name,password,password,['Manager'],0)

    #clean up code
    self.delSessionExpertFormData()
    self.delSessionErrors()

    # sends notification email
    self.registrationEmail(Email, self.adminEmail, Firstname + ' ' + Lastname, name, password, self.getEngine().absolute_url(0) + '/index_html')

    # redirect
    if REQUEST is not None:
        if getSecurityManager().checkPermission('View management screens', self):
            return self.manage_main(self, REQUEST, update_menu=1)
        else:
            ob.autoLoginUser(name, password, REQUEST)
            return REQUEST.RESPONSE.redirect(ob.absolute_url(0) + '/editprofile_html')

class Expert(SimpleItem, ExpertData):
    """ Expert object - a Zope user with a propertysheet holding user's credentials """

    meta_type = Resources.METATYPE_EXPERT
    icon = 'misc_/BesserWisser/expert.gif'

    security = ClassSecurityInfo()

    manage_options = (
        (
            {'label':'Credentials', 'action':'manage_credentials'},
            {'label':'Expertise', 'action':'manage_expertise'},
            {'label':'View', 'action':'index_html'},
            {'label':'Picture', 'action':'manage_image'},
        )
        +
        SimpleItem.manage_options
    )

    option_tabs = (
        {'label':'Answer questions', 'action':'answerquestions_html'},
        {'label':'Edit profile','action':'editprofile_html'},
        {'label':'Change password', 'action':'changepassword_html'},
        {'label':'View your profile', 'action':'index_html'},
        {'label':'View answers', 'action':'listanswers_html'},
        {'label':'Enrol for categories', 'action':'enroleforcategories_html'},
    )

    def __init__(self, name, password, Firstname, Lastname, Email, ProfileNativeIsIn, LanguagesUnderstood, 
        Nationality, ProfileEnglish, ProfileNative, Employer,Expertise, QuestionsPerDay, isOrganisation, 
        Picture, onVacation, ExpectedReturnDate):
        """ Expert constructor """
        self.id = name
        self.title = Firstname + ' ' + Lastname
        self.expert_picture = None
        ExpertData.__dict__['__init__'](self, Firstname, Lastname, Nationality, Email, ProfileEnglish, 
            ProfileNative, Employer, Expertise, LanguagesUnderstood, QuestionsPerDay, ProfileNativeIsIn, 
            isOrganisation, Picture, onVacation, ExpectedReturnDate)

    ################################
    ## Callable methods            #
    ################################
    security.declarePrivate('checkExpertId')
    def checkExpertId(self):
        """ Checks if current authenticated user is the current expert """
        return self.id == self.REQUEST.AUTHENTICATED_USER.getUserName()

    security.declarePrivate('allAnsweredQuestions')
    def allAnsweredQuestions(self):
        """ Counts answered questions - all times """
        return len(self.getAnsweredQuestionsForExpert(self.id, filterPrivate=1, filterAnswered=1, answeredRangeMin=None))

    security.declarePrivate('allAskedQuestions')
    def allAskedQuestions(self):
        """ Returns a list with all questions for this expert """
        return self.getAskedQuestionsForExpert(self.id)

    security.declarePrivate('lastYearAnsweredQuestions')
    def lastYearAnsweredQuestions(self):
        """ Counts answered questions - past yesr """
        return self.getAnsweredQuestionsForExpert(self.id, filterPrivate=None, filterAnswered=1, answeredRangeMin=365)

    security.declarePrivate('questionsToday')
    def questionsToday(self):
        """ Counts the questions for the expert in the current day """
        return len(self.getAskedQuestionsForExpertToday(self.id))

    security.declareProtected(view, 'canBeAskedInLanguage')
    def canBeAskedInLanguage(self, p_language_id):
        """ Tests if the expert can be asked in the given language """
        # check if there is space for another question
        if self.questionsToday() >= self.QuestionsPerDay:
            return 0
        else:
            # check for a valid language id for this expert
            if p_language_id not in self.LanguagesUnderstood:
                return 0
        return 1

    def averageRating_Timelineness(self, p_filterPrivate=None): 
        """ Calculates the average timelineness and ratings for an expert
            Used to display expert statistics - topExperts """
        # sort answered expert questions by date answered
        l_expert_questions = self.utSortObjsListByAttr(self.getAnsweredQuestionsForExpert(self.id, filterPrivate=p_filterPrivate, filterAnswered=1, answeredRangeMin=None), 'answeredDateTime', 1)
        texts = ['not available','weak','nearly weak','normal','good','best']
        ar = 0 # sum of ratings
        at = 0 # sum of timelineness
        rra = 0 # number of questions taken into account for rating = answered and rated
        rrt = 0 # number of questions taken into account for timelineness = all public questions
                # for the unanswered questions, the timelineness is calculated as if the question would be
                # answered in that very second
        rr = 0 # avarage rating
        rt = 0 # avarage timelineness
        rrya = 0 # number of answered questions in the last year
        rry = 0 # ratings + timeliness in the last year
        rrma = 0 # number of answered questions in the last month
        rrm = 0 # ratings + timeliness in the last month
        rrwa = 0 # number of answered questions in the last week
        rrw = 0 # ratings + timeliness in the last week
        rr5a = 0 # number of answered questions in the 3 days
        rr5 = 0 # ratings + timeliness in the last 5 questions
        for l_expert_question in l_expert_questions:
            if l_expert_question.answeredDateTime.greaterThan(DateTime('01/01/1990')):
                counter = 0
                rra = rra + 1
                rrt = rrt + 1
                try: r = float(l_expert_question.Rating)
                except: r = 0
                try: t = float(l_expert_question.Timelineness)
                except: t = 0
                # Disregard unrated questions
                if r == 0: rra = rra - 1
                else: ar = ar + r
                at = at + t
                # last (at most) 5 answered questions
                if counter < 5:
                    if r > 0:
                        counter += 1
                        rr5a += 1
                        rr5 += r
            # give expert 2 weeks to answer, so calculate timelineness as if the question would
            # be answered in this second
            elif DateTime() - l_expert_question.askedDateTime < 15:
                rrt = rrt + 1
                diff = DateTime() - l_expert_question.askedDateTime
                if diff < 1:
                    t = t + 5
                elif diff > 1 and diff < 1.5:
                    t = t + 4
                elif diff > 1.5 and diff < 2 :
                    t = t + 3
                elif diff > 2 and diff < 3:
                    t = t + 2
                elif diff > 3:
                    t = t + 1
                ar = ar + t
            # if there have been more then 2 weeks, count the question and consider timelineness 0
            else:
                rrt = rrt + 1
            # calculate number of questions in the last year, last month and last week
            ddif = DateTime() - l_expert_question.answeredDateTime
            if ddif < 365:
                rrya = rrya + 1
                rry = rry + t
            if ddif < 31:
                rrma = rrma + 1
                rrm = rrm + t
            if ddif < 8:
                rrwa = rrwa + 1
                rrw = rrw + t
        # calculate average rating 
        if rra: rr = ar/rra
        else: rr = 0
        # calculate average timelineness
        if rrt: rt = at/rrt
        else: rt = 0
        # calculate average in the last year and put value in REQUEST
        if rrya: ry = rry/(rrya)
        else: ry = 0
        # calculate average in the last month and put value in REQUEST
        if rrma: rm = rrm/(rrma)
        else: rm = 0
        # calculate average in the last week and put value in REQUEST
        if rrwa: rw = rrw/(rrwa)
        else: rw = 0
        # calculate average in the last 5 questions and put value in REQUEST
        if rr5a: r5 = rr5/(rr5a)
        else: r5 = 0
        return (rra, rr, texts[int(round(rr))], rt, texts[int(round(rt))], ry, texts[int(round(ry))], rm, texts[int(round(rm))], rw, texts[int(round(rw))], r5, texts[int(round(r5))])

    security.declarePrivate(view, 'averageEstimateTimelineness')
    def averageEstimateTimelineness(self):
        """ 
        Calculates the average timelineness for an expert:
            Take all the answered questions and the ones unanswered for at most
            2 weeks and calculate de timelineness
        Used to display expert statistics - topExperts
        """
        allPublicForExpert = self.getAnsweredQuestionsForExpert(self.id, filterPrivate=1, filterAnswered=None, answeredRangeMin=None)
        sumAnsweredTimeliness = 0
        nrAnswered = 0
        sumUnansweredTimeliness = 0
        nrUnanswered = 0
        for quest in allPublicForExpert:
            if quest.answeredDateTime:
                try: sumAnsweredTimeliness += int(quest.Timelineness)
                except: pass
                nrAnswered += 1
            # Give the expert 2 weeks time to answer and rate the question 
            # as if it was answered in that very moment
            elif quest.askedDateTime.greaterThan(DateTime() - 14):
                try: sumUnansweredTimeliness += int(quest.Timelineness)
                except: pass
                nrUnanswered += 1
        ret = 0
        if nrAnswered > 0:
            ret = sumAnsweredTimeliness / nrAnswered
        if nrUnanswered > 0:
            ret += sumUnansweredTimeliness / nrUnanswered
        return ret

    def calculateRatingPicture(self, nAnswers):
        """ Returns the path to the star picture given for total answers to an expert """
        if nAnswers < 5:
            star_img = 'greystar'
        elif nAnswers >= 5 and nAnswers < 9:
            star_img = 'yellowstar'
        elif nAnswers >= 9 and nAnswers < 12:
            star_img = 'bluestar'
        elif nAnswers >= 12 and nAnswers < 15:
            star_img = 'turqstar'
        elif nAnswers >= 15 and nAnswers < 18:
            star_img = 'purplestar'
        elif nAnswers >= 18 and nAnswers < 20:
            star_img = 'redstar'
        elif nAnswers >= 20 and nAnswers < 22:
            star_img = 'greenstar'
        elif nAnswers >= 22 and nAnswers < 24:
            star_img = 'yellowshoot'
        elif nAnswers >= 24 and nAnswers < 26:
            star_img = 'turqshoot'
        elif nAnswers >= 26 and nAnswers < 30:
            star_img = 'purpleshoot'
        elif nAnswers >= 30:
            star_img = 'redshoot'
        star_img = '/misc_/BesserWisser/' + star_img + '.gif'

        return star_img

    ################################
    ## Special management methods  #
    ################################

    def manage_afterAdd(self, item, container, new_fn=None):
        """ This method is called, whenever _setObject in ObjectManager gets
        called. This is the case after a normal add and if the object is a
        result of cut-paste- or rename-operation. """
        self.catalogBesserWisserObject(self)
        Expert.inheritedAttribute('manage_afterAdd')(self, item, container)

    def manage_beforeDelete(self, item, container):
        """ This method is called, when the object is deleted. """
        try:
            del self.acl_users.data[item.id]
        except:
            pass
        self.uncatalogBesserWisserObject(self)
        self.updateExpertExpertise(self.Expertise, [])
        Expert.inheritedAttribute('manage_beforeDelete')(self, item, container)

    def manage_undo_transactions(self, transaction_info, REQUEST=None):
        """ This method is called, when the user has chosen an Undo-action. """
        try:
            self.acl_users._addUser(self.id,self.password,self.password,['Manager'],0)
        except:
            pass
        return Expert.inheritedAttribute ("manage_undo_transactions") (self, transaction_info, REQUEST)

    def manageExpertise(self, Expertise=[], REQUEST=None):
        """ The Expertise tab """
        old_expertise = self.Expertise
        new_expertise = self.utConvertToList(Expertise)
        self.Expertise = new_expertise
        self.updateExpertExpertise(old_expertise, new_expertise)
        self._p_changed = 1
        self.catalogBesserWisserObject(self)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect('manage_expertise')

    def manage_uploadExpertPicture(self, file='', REQUEST=None):
        """ Uploads the picture """
        self.setExpertPicture(file)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect('manage_image')

    def manage_editCredentials(self, Firstname, Lastname, Email, Employer, 
            Nationality, ProfileNativeIsIn, ProfileEnglish, ProfileNative, QuestionsPerDay=5,
            LanguagesUnderstood = [], onVacation=0,  isOrganisation=0, ExpectedReturnDate=None, REQUEST=None):
        """ The Credentials tab in ZMI """
        self.title = Firstname + ' ' + Lastname
        self.Firstname = Firstname
        self.Lastname = Lastname
        self.Email = Email
        self.Employer = Employer
        try:
            self.QuestionsPerDay = int(QuestionsPerDay)
        except:
            self.QuestionsPerDay = 5
        if isOrganisation:
            self.isOrganisation = 1
        else:
            self.isOrganisation = 0
        self.Nationality = Nationality
        self.LanguagesUnderstood = LanguagesUnderstood
        self.ProfileNativeIsIn = ProfileNativeIsIn
        self.ProfileEnglish = ProfileEnglish
        self.ProfileNative = ProfileNative
        self.onVacation = onVacation
        if self.onVacation:
            try: self.ExpectedReturnDate = DateTime(ExpectedReturnDate)
            except: self.ExpectedReturnDate = DateTime((DateTime() + 1).Date())
        else:
            self.ExpectedReturnDate = None
        self.recatalogBesserWisserObject(self)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect('manage_credentials')

    def updateExpertExpertise(self, old_expertise, new_expertise):
        """ Add this expert to each category in expertise list """
        add_expertise = self.utListDifference(new_expertise, old_expertise)
        del_expertise = self.utListDifference(old_expertise, new_expertise)
        for item in add_expertise:
            try:
                category = self.unrestrictedTraverse(item)
                category.enroledExperts.append(self.id)
                category._p_changed = 1
            except:
                pass
        for item in del_expertise:
            try:
                category = self.unrestrictedTraverse(item)
                category.enroledExperts.remove(self.id)
                category._p_changed = 1
            except:
                pass
        #update expertise history
        self.Expertise_history_enroled = add_expertise
        self.Expertise_history_disabled = del_expertise

    security.declarePrivate('setExpertPicture')
    def setExpertPicture(self, p_picture):
        """ set picture """
        if p_picture != '':
            if p_picture.filename:
                headers = getattr(p_picture, 'headers', None)
                if headers and headers.has_key('content-type'):
                    content_type = headers['content-type']
                self.expert_picture = ExpertImage(p_picture, content_type)
                self._p_changed = 1

    def getExpertPicture(self):
        """ return the expert picture """
        return self.expert_picture.data

    def getExpertLanguagesUnderstoodList(self):
        """ """
        return ', '.join(map(self.getLanguageName, self.LanguagesUnderstood))

    def editProfile(self, Firstname='', Lastname='', Email='', Nationality='', Employer='',
            ProfileEnglish='', ProfileNativeIsIn='', ProfileNative='', LanguagesUnderstood=[],
            QuestionsPerDay=0, Picture='', onVacation=0, ExpectedReturnDate=None, REQUEST=None):
        """ """
        l_errors = []
        #convertions
        LanguagesUnderstood = self.utConvertToList(LanguagesUnderstood)
        #validate form data
        if not Firstname:
            l_errors.append(Resources.ERR_MSG_FIRSTNAME_REQUIRED)
        if not Lastname:
            l_errors.append(Resources.ERR_MSG_LASTNAME_REQUIRED)
        if not Email:
            l_errors.append(Resources.ERR_MSG_EMAIL_REQUIRED)
        #load ExpertFormData object and put all the data in there
        l_expertformdata = self.getSessionExpertFormData(self.getEmptyExpertFormData())
        l_expertformdata.update(Firstname, Lastname, Email, ProfileNativeIsIn, LanguagesUnderstood, Nationality, self.id, '', [], Employer, ProfileEnglish, ProfileNative, QuestionsPerDay, onVacation, ExpectedReturnDate)
        #check for errors
        if len(l_errors)>0:
            if REQUEST:
                self.setSessionExpertFormData(l_expertformdata)
                self.setSessionErrors(l_errors)
                return REQUEST.RESPONSE.redirect('editprofile_html')
            else:
                return None
        else:
            self.title = Firstname + ' ' + Lastname
            self.Firstname = Firstname
            self.Lastname = Lastname
            self.Email = Email
            self.Nationality = Nationality
            self.Employer = Employer
            self.ProfileEnglish = ProfileEnglish
            self.ProfileNativeIsIn = ProfileNativeIsIn
            self.ProfileNative = ProfileNative
            self.LanguagesUnderstood = LanguagesUnderstood
            try:
                self.QuestionsPerDay = int(QuestionsPerDay)
            except:
                self.QuestionsPerDay = 5
            self.setExpertPicture(Picture)
            self.onVacation = onVacation
            if self.onVacation:
                try: self.ExpectedReturnDate = DateTime(ExpectedReturnDate)
                except: self.ExpectedReturnDate = DateTime((DateTime() + 1).Date())
            else:
                self.ExpectedReturnDate = None
            self.recatalogBesserWisserObject(self)
            self.delSessionErrors()
            if REQUEST:
                return REQUEST.RESPONSE.redirect('editprofile_html')

    def changePassword(self, expert_password, expert_confirmpassword, REQUEST):
        """ """
        l_errors = []
        #validate parameters
        if not expert_password:
            l_errors.append(Resources.ERR_MSG_PASSWORD_REQUIRED)
        else:
            if expert_password != expert_confirmpassword:
                l_errors.append(Resources.ERR_MSG_PASSWORDS_MUST_MATCH)
        #check for errors
        if len(l_errors)>0:
            l_errors.append(Resources.ERR_MSG_RETYPE_PASSWORDS)
            if REQUEST:
                self.setSessionErrors(l_errors)
                return REQUEST.RESPONSE.redirect('changepassword_html')
            else:
                return None
        else:
            self.delSessionErrors()
            #change password
            acl_user_obj = self.getExpertUser(REQUEST.AUTHENTICATED_USER.getUserName())
            acl_user_obj.__ = expert_password
            self.autoLoginUser(acl_user_obj.name, acl_user_obj.__, REQUEST)
            if REQUEST:
                REQUEST.RESPONSE.redirect('changepassword_html?changed=1')

    security.declarePrivate('autoLoginUser')
    def autoLoginUser(self, p_user_name, p_user_password, REQUEST):
        """ auto login user """
        from base64 import encodestring
        from urllib import quote
        ac = encodestring('%s:%s' % (p_user_name, p_user_password))
        REQUEST._auth = 'basic %s' % ac
        REQUEST.RESPONSE._auth = 1
        cookie_name = BWCookieCrumbler.auth_cookie
        cookie_value = quote(ac)
        REQUEST.RESPONSE.setCookie(cookie_name, cookie_value, path='/')

    def getEnroledCategories(self):
        """ Returns a list of Categories objects """
        return map(self.unrestrictedTraverse, self.Expertise)

    def getHistoryEnroledCategories(self):
        """ Returns a list of Categories objects """
        return map(self.unrestrictedTraverse, self.Expertise_history_enroled)

    def getHistoryDisabledCategories(self):
        """ Returns a list of Categories objects """
        return map(self.unrestrictedTraverse, self.Expertise_history_disabled)

    def enroleForCategories(self, Expertise=[], REQUEST=None):
        """ Enrole Experts for some Categories """
        old_expertise = self.Expertise
        new_expertise = self.utConvertToList(Expertise)
        self.Expertise = new_expertise
        self.updateExpertExpertise(old_expertise, new_expertise)
        self._p_changed = 1
        self.catalogBesserWisserObject(self)
        if REQUEST:
            REQUEST.RESPONSE.redirect('enroleforcategories_html?enroled=1')

    manage_credentials = Globals.DTMLFile("dtml/Expert_manageCredentials", globals())
    manage_expertise = Globals.DTMLFile("dtml/Expert_manageExpertise", globals())
    manage_image = Globals.DTMLFile("dtml/Expert_manageImage", globals())

    security.declareProtected(view, 'index_html')
    index_html = Globals.DTMLFile('dtml/Expert_showProfile', globals())

    security.declareProtected(view, 'answerquestions_html')
    answerquestions_html = Globals.DTMLFile('dtml/Expert_answerQuestions', globals())

    security.declareProtected(view, 'editprofile_html')
    editprofile_html = Globals.DTMLFile('dtml/Expert_editProfile', globals())

    security.declareProtected(view, 'changepassword_html')
    changepassword_html = Globals.DTMLFile('dtml/Expert_changePassword', globals())

    security.declareProtected(view, 'enroleforcategories_html')
    enroleforcategories_html = Globals.DTMLFile('dtml/Expert_enroleForCategories', globals())

    security.declareProtected(view, 'listanswers_html')
    listanswers_html = Globals.DTMLFile('dtml/Expert_listanswers', globals())

Globals.InitializeClass(Expert)

class ExpertImage:
    def __init__(self, data, content_type):
        """ Holds the Expert picture """
        self.data = data.read()
        self.content_type = content_type
        self.size = len(self.data)
